#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "proto.h"
#include "channel.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef _WIN32
#include <io.h>
#endif
#include <fcntl.h>
#include "h.h"
#ifdef STRIPBADWORDS
#include "badwords.h"
#endif
#ifdef _WIN32
#include "version.h"
#endif

ModuleHeader Mod_Header
  = {
    "m_scarecrow",  /* Name of module */
    "v0.0.7", /* Version */
    "Scarecrow - http://ircdxp.sf.net", /* Short description of module */
    "3.2-b8-1",
    NULL 
    };
/*
 * CONFIGURATION PART
 */
#define EXP_MIRC_DCC					1 
/* [RECOMMENDED] enables protection against the mirc exploit */
#define TRJ_ANATOLY_FYLE_BOUNCE_TEXT	1 
/* [OPTIONAL] sends a bounce when a suspicious text line is detected */
#define TRJ_ANATOLY_FYLE_BOUNCE_GLINE	0
/* [DISCOURAGED] sends a gline when a suspicious text line detected */
#define TRJ_ANATOLY_FYLE_EARLY_BOUNCE	1
/* [RECOMMENDED] sends a bounce when a suspicious nickserv register line is detected */
/* This will bounce the Trojan before even letting it to join a single channel */
/* Note: won't work if no nickserv is online, also no notice will be displayed */
/************************/


static ModuleInfo ADCMod;

static Hook *UserMsg = NULL, *ChanMsg = NULL;

DLLFUNC char *adc_usermsg(aClient *, aClient *, aClient *, char *, int);
DLLFUNC char *adc_chanmsg(aClient *, aClient *, aChannel *, char *, int);

/* This is called on module init, before Server Ready */
DLLFUNC int Mod_Init(ModuleInfo *modinfo)
{
    bcopy(modinfo,&ADCMod,modinfo->size);
    UserMsg = HookAddPCharEx(ADCMod.handle, HOOKTYPE_USERMSG, adc_usermsg);
    ChanMsg = HookAddPCharEx(ADCMod.handle, HOOKTYPE_CHANMSG, adc_chanmsg);
    return MOD_SUCCESS;
}

DLLFUNC int Mod_Load(int module_load)
{
    return MOD_SUCCESS;
}

DLLFUNC int Mod_Unload(int module_unload)
{
    HookDel(UserMsg);
    HookDel(ChanMsg);
    return MOD_SUCCESS;
}

DLLFUNC char *adc_usermsg(aClient *cptr, aClient *sptr, aClient *acptr, char *text, int notice)
{
	if (!text)
	{
		return text;
	}

    if (EXP_MIRC_DCC && (text[0] == '\001') && !strncasecmp(text+1, "DCC ", 4))
    {
        int spaces=0;
        char *p;
        for (p = text; *p; p++)
            if (*p == ' ')
                spaces++;
        if ((spaces > 20) || (strlen(text) > 200))
        {
            sendto_one(sptr, ":%s NOTICE %s :Suspicious DCC command detected (possible mIRC exploit). "
                                 "Opers have been notified.", me.name, sptr->name);
            if (MyClient(sptr))
            {
                sptr->since += 7;
                sptr->flags |= FLAGS_DCCBLOCK;
            }
            sendto_realops("from %s: (EXP/mirc.dcc) %s %s %s", me.name, sptr->name, sptr->user->username, sptr->user->realhost);
            sendto_serv_butone(cptr, ":%s GLOBOPS :(EXP/mirc.dcc) %s %s %s", me.name, sptr->name, sptr->user->username, sptr->user->realhost);
            return NULL;
        }
    }

	if (TRJ_ANATOLY_FYLE_BOUNCE_TEXT &&
		!strncmp(text, "Come watch me on my webcam and chat /w me :-) http://", 53) &&
		!strncmp(text+(strlen(text)-7), "/me.mpg", 7))
    {
        sendto_realops("from %s: (TRJ/anatoly.fyle) %s %s %s", me.name, sptr->name, sptr->user->username, sptr->user->realhost);
        sendto_serv_butone(cptr, ":%s GLOBOPS :(TRJ/anatoly.fyle) %s %s %s", me.name, sptr->name, sptr->user->username, sptr->user->realhost);
		sendto_one(sptr, ":%s 439 %s :Dear user, please ignore this, you are being scanned for a known trojan.", me.name, sptr->name);
        if (TRJ_ANATOLY_FYLE_BOUNCE_GLINE && !IsOper(sptr)) {
            char mo[1024], mo2[1024];
            char *tkllayer[9] = { me.name, "+", "G", "*", sptr->user->realhost, "scarecrow!sentinel@ircdxp.sf.net", "0", NULL, "(TRJ/anatoly.fyle) Trojan Drone Detected" }; 
            tkl_check_expire(NULL);
            ircsprintf(mo, "%li", 345600 + TStime());
            ircsprintf(mo2, "%li", TStime());
            tkllayer[6] = mo;
            tkllayer[7] = mo2;
            m_tkl(&me, &me, 9, tkllayer);
        }
        return NULL;
    }
    
    if (TRJ_ANATOLY_FYLE_EARLY_BOUNCE &&
		!strncmp(text, "register ", 9) &&
        !strncmp(text+(strlen(text)-12), "@hotmail.com", 12) &&
        !strcasecmp(acptr->name, "nickserv"))
    {
        sendto_one(sptr, ":%s 439 %s :Dear user, please ignore this, you are being scanned for a known trojan.", me.name, sptr->name);
    }

    return text;
}

DLLFUNC char *adc_chanmsg(aClient *cptr, aClient *sptr, aChannel *chptr, char *text, int notice)
{
    if (text && (text[0] == '\001') && !strncasecmp(text+1, "DCC ", 4))
    {
        int spaces=0;
        char *p;
        for (p = text; *p; p++)
            if (*p == ' ')
                spaces++;
        if ((spaces > 20) || (strlen(text) > 200))
        {
            sendto_one(sptr, ":%s NOTICE %s :Suspicious DCC command detected (possible mIRC exploit). "
                             "Opers have been notified.", me.name, sptr->name);
            if (MyClient(sptr))
            {
                sptr->since += 7;
                sptr->flags |= FLAGS_DCCBLOCK;
            }
            sendto_realops("from %s: (EXP/mirc.dcc) %s %s %s", me.name, sptr->name, sptr->user->username, sptr->user->realhost);
            sendto_serv_butone(cptr, ":%s GLOBOPS :(EXP/mirc.dcc) %s %s %s", me.name, sptr->name, sptr->user->username, sptr->user->realhost);
            return NULL;
        }
    }
    return text;
}

